#include <minix/config.h>
#include <sys/types.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <fcntl.h>
#include <sgtty.h>
#include <sys/fdio.h>
#include <stdio.h>

#define tl(upm, tr)	((6250L * tr * 300L)/(upm * 250L))
int verbose = 0;

_PROTOTYPE(int gap3 , (int spt , int seclen , int trklen , int iso , int g4 ));
_PROTOTYPE(int buildtrack , (char *data , char *sectab , int spt , int track , int side , int virgin , int seclen , int trklen , int iso , int g4 ));
_PROTOTYPE(int main , (int argc , char **argv ));
_PROTOTYPE(void usage , (char *prg ));

int gap3(spt, seclen, trklen, iso, g4)/* computes the optimum size of gap3    */
int spt, seclen, trklen, iso, g4;     /* returns -1 for error		      */
				      /* a rotation speed variation of 2% is  */
{				      /* assumed			      */
	int gap3min;
	int gap3max;
	int gap4min;

	gap4min = (trklen / g4);
	if (iso) {
		gap3max = (trklen - gap4min - 32)/spt - (seclen + 62);
	} else {
		gap3max = (trklen - gap4min - 66)/spt - (seclen + 62);
	}
	gap3min = (seclen + 62)/50;

	return((gap3max < gap3min) ? -1:gap3max);
}

int buildtrack(data, sectab, spt, track, side, virgin, seclen, trklen, iso, g4)
char *data, *sectab; int spt, track, side, virgin, seclen, trklen, iso, g4;
{
	int gap3len, i;
	register char *cp;

	cp = data;
	if ((gap3len = gap3(spt, seclen, trklen, iso, g4)) < 0)
		return(0);

	/*    Track header    */
	if (iso) {
		memset(cp, 0x4e, 32); cp += 32; /* gap 1	*/
	} else {
		memset(cp, 0x4e, 80); cp += 80; /* gap 4a	*/
		memset(cp, 0x00, 12); cp += 12; /* sync 	*/
		memset(cp, 0xf6,  3); cp +=  3; /* C2		*/
		*cp++ = 0xfc;			/* index mark	*/
		memset(cp, 0x4e, 50); cp += 50; /* gap 1	*/
	}

	for(i = 0; i < spt; i++)
	{
		/*	 sync	    */
		memset(cp, 0x00, 12); cp += 12;

		/* id-address  mark  */
		memset(cp, 0xf5, 3); cp += 3;
		*cp++ = 0xfe;

		/*     id-field     */
		*cp++ = track;
		*cp++ = side;
		*cp++ = sectab[i];
		switch(seclen)
		{
			case 128:
				*cp++ = 0;
				break;

			case 256:
				*cp++ = 1;
				break;

			case 512:
				*cp++ = 2;
				break;

			case 1024:
				*cp++ = 3;
				break;

			case 2048:
				*cp++ = 4;
				break;

			case 4096:
				*cp++ = 5;
				break;

			default:
				fprintf(stderr, "illegal sector size %d\n", seclen);
				exit(1);
		}

		/* write  CRC- code  */
		*cp++ = 0xf7;

		/*	 gap2	    */
		memset(cp, 0x4e, 22); cp += 22;
		memset(cp, 0x00, 12); cp += 12;

		/* data address mark */
		memset(cp, 0xf5, 3); cp += 3;
		*cp++ = 0xfb;

		/*    data field    */
		memset(cp, virgin, seclen);
		cp += seclen;

		/* write  CRC- code */
		*cp++ = 0xf7;

		/*	gap 3	    */
		memset(cp, 0x4e, gap3len); cp += gap3len;
	}

	/*	 gap 4	      */
	memset(cp, 0x4e, (int)(trklen - (cp - data)));
	return(1);
}

void usage(prg)
char *prg;
{
	fprintf(stderr, "\
Usage: %s [-v] [-q] [-w] [-i n] [-s n] [-ISO|-IBM|-G n] device\n\
       -v    Verbose operation\n\
       -q    Quiet operation\n\
       -w    Wipe disk\n\
       -i    Format with interleave n\n\
       -s    Format with spiralisation n\n\
       -ISO  Format using ISO trackheader\n\
       -IBM  Format using IBM trackheader\n\
       -G    Format using ISO trackheader\n\
             GAP4 is set to n %% of the tracksize\n", prg);
}

int main(argc, argv)
int argc; char **argv;
{
	register int i, j, sides, trklen;
	register char *p;
	int fd;
	struct fparam fmtparam;
	struct fparam rawparam;

	extern char *optarg;
	extern int optind;
	int qflag = 0;		/* quiet mode		*/
	int wflag = 0;		/* wipe disk		*/
	int interl = 1;		/* sector interleave	*/
	int spiral = 1;		/* track interleave	*/
	int iso = 1;		/* iso or ibm format	*/
	int gap4 = 50;		/* Gap 4 len in 100/n%	*/
	int c;
	char dev1[64], dev2[64];

	char t[30], tc[30];

	while ((c = getopt(argc, argv, "qvwi:s:G:I:")) != EOF) {
		switch (c) {
			/* quiet */
			case 'q':
				qflag++;
				break;

			/* verbose */
			case 'v':
				verbose++;
				break;

			/* set interleave */
			case 'i':
				interl = atoi(optarg);
				break;

			/* set spiral factor */
			case 's':
				spiral = atoi(optarg);
				break;

			case 'w':
				wflag++;
				break;

			case 'G':
				gap4 = atoi(optarg);
				if (gap4 < 0 || gap4 > 100)
					usage(argv[0]);
				if (gap4 == 0)
					gap4 = 30000;
				else
					gap4 = 100 / gap4;
				iso = 1;
				break;

			case 'I':
				if (strcmp(optarg, "SO") == 0) {
					iso = 1;
					break;
				} else if (strcmp(optarg, "BM") == 0) {
					iso = 0;
					break;
				}
				/* FALL THROUGH */
				
			default:
				usage(argv[0]);
				exit(1);
		}
	}

	if (!argv[optind]) {
		usage(argv[0]);
		exit(1);
	}

	*dev2 = '\0';
	if (!strncmp(argv[optind], "/dev/r", 6))
		strcpy(dev1, argv[optind]);
	else if (!strncmp(argv[optind], "/dev/dsk/", 9))
		sprintf(dev1, "/dev/rdsk/r%s", argv[optind] + 9);
	else if (!strncmp(argv[optind], "/dev/", 5))
		sprintf(dev1, "/dev/r%s", argv[optind] + 5);
	else if (*argv[optind] == 'r') {
		sprintf(dev1, "/dev/%s", argv[optind]);
		sprintf(dev2, "/dev/rdsk/%s", argv[optind]);
	} else if (*argv[optind] != '/') {
		sprintf(dev1, "/dev/r%s", argv[optind]);
		sprintf(dev2, "/dev/rdsk/r%s", argv[optind]);
	} else
		strcpy(dev1, argv[optind]);

	if (!*dev2)
		strcpy(dev2, dev1);

	/* open the device */
	if (((fd = open(dev1, O_WRONLY)) < 0  &&
	     (fd = open(dev2, O_WRONLY)) < 0) ||
	    /* get format description */
	    ioctl(fd, DIOGETP, (struct sgttyb *)&rawparam) < 0 ||
	    /* lock the drive */
	    ioctl(fd, DIOFMTLOCK, (struct sgttyb *)&fd) < 0    ||
	    /* get the format description for formatting */
	    ioctl(fd, DIOGETP, (struct sgttyb *)&fmtparam) < 0) {
		perror(argv[0]);
		exit(1);
	}

	switch (fmtparam.sector_size) {
	case  6656: trklen = tl(300L, 250L);	break;
	case  8192: trklen = tl(300L, 320L);	break;
	case 10752: trklen = tl(360L, 500L);	break;
	case 12800: trklen = tl(300L, 500L);	break;
	default:    trklen = fmtparam.sector_size; break;
	}

	sides = rawparam.nr_sides + 1;

	/* build interleave vector */
	for (i = 0; i < rawparam.nr_sectors; i++)
		t[i] = -1;

	for (i = 0, j = 0; i < rawparam.nr_sectors; i++)
	{
		while (t[j] != -1)
			j = (j+1) % rawparam.nr_sectors;

		t[j] = i + rawparam.sector_0;
		j = (j+interl) % rawparam.nr_sectors;
	}

	if (!wflag && verbose)
	{
		printf("Sector sequence: ");
		for (i = 0; i < rawparam.nr_sectors; i++)
			printf(" %2d ", t[i]);
		printf("\n");
		fprintf(stderr, "\
Sector len = %d\n\
GAP3   len = %d\n\
Track  len = %d\n\
GAP4   len = %d\n\n\
Total formatted capacity = %ld bytes\n",
		rawparam.sector_size,
		gap3(rawparam.nr_sectors, rawparam.sector_size, trklen,
		     iso, gap4),
		trklen, trklen/gap4,
		(long)rawparam.nr_sectors * rawparam.sector_size *
					rawparam.nr_cylinders * sides);
	}

	p = (char *)malloc(fmtparam.sector_size);
	if (!p) {
		fprintf(stderr, "Out of memory\n");
		exit(2);
	}
	memset(p, 0, fmtparam.sector_size);

	/* format disk */
	for (i = rawparam.nr_cylinders - 1; i >= 0; i--)
	{
		if (lseek(fd, (long)fmtparam.sector_size * i * sides, SEEK_SET) !=
		    (long)fmtparam.sector_size * i * sides) {
			fprintf(stderr, "\n");
			perror(argv[0]);
			exit(1);
		}

		for (j = 0; j < sides; j++)
		{
			if (!wflag &&
			    !buildtrack(p, t, rawparam.nr_sectors, i, j, 0xe5,
					rawparam.sector_size, trklen, iso, gap4))
			{
				fprintf(stderr, "\n%s: track too long\n", argv[0]);
				fflush(stdout);
				exit(1);
			}

			if (!qflag) {
				if (wflag)
					printf("Wiping track %02d side %d\r", i, j);
				else
					printf("Formatting track %02d side %d\r", i, j);
				fflush(stdout);
			}

			if (write(fd, p, fmtparam.sector_size) <
			    fmtparam.sector_size)
			{
				fprintf(stderr, "\n");
				perror(argv[0]);
				exit(1);
			}
		}

		for (j = 0; j < rawparam.nr_sectors; j++)
			tc[(j + rawparam.nr_sectors - spiral)%rawparam.nr_sectors] = t[j];

		for (j = 0; j < rawparam.nr_sectors; j++)
			t[j] = tc[j];
	}
	if (!qflag)
		printf("\n");
	exit(0);
}
