#include <minix/config.h>
#include <minix/const.h>
#if (CHIP == M68000)
|
| This file contains a number of assembly code utility routines needed by the
| 68000 versions of the kernel. Basically they are for copying data.
|
| All the routines in here destroy d0,d1,a0,a1. All other regs are preserved.
|
|
| Edition history:
|  #    Date                         Comments                       By
| --- -------- ---------------------------------------------------- --- 
|   1 13.06.89 fast phys_copy by Dale Schumacher                    DAL
|   2 16.06.89 bug fixes and code impromvement by Klamer Schutte    KS
|   3 12.07.89 bug fix and further code improvement to phys_copy    RAL
|   4 14.07.89 flipclicks,zeroclicks,copyclicks added               RAL
|   5 15.07.89 fast copy routine for messages added to phys_copy    RAL
|   6 03.08.89 clr.l <ea> changed to move.l #0,<ea> (= moveq )      RAL
|   7 18.04.91 support for CLICK_SIZE <> 256 added		    KUB
|   8 19.04.91 changes all over the code (for readability 8-)       KUB
|   9 20.04.91 phys_copy improved; jump into table instead of dbra  KUB

#if CLICK_SIZE < 256
#error					/* Only CLICK_SIZE >= 256 supported! */
#endif
#define CSHIFT	CLICK_SHIFT		/* a problem with gcc-gas and cv68 */

|
| public labels
|
.globl	_copyclicks		| copy a block of n clicks
.globl	_flipclicks		| exchange 2 blocks of n clicks
.globl	_zeroclicks		| zero a block of n clicks
.globl	_phys_copy		| copy a block of n bytes

.text

|*===========================================================================*
|*				copyclicks				     *
|*===========================================================================*
|PUBLIC void copyclicks(phys_clicks src, phys_clicks dest, phys_clicks nclicks)

#define	cc_src	4(sp)
#define	cc_dest	6(sp)
#define	cc_ncl	8(sp)

_copyclicks:
	movel	#CSHIFT,d1

	movel	#0,d0
	movew	cc_src,d0		| source address in clicks
	lsll	d1,d0
	movel	d0,a0		| source address

	movel	#0,d0
	movew	cc_dest,d0		| destination address in clicks
	lsll	d1,d0
	movel	d0,a1		| destination address

	movel	#0,d0
	movew	cc_ncl,d0		| number of clicks
	beq	cc_ret		| nothing to copy

	moveml	d2-d7/a2-a5,sp@-		| save registers

#if CLICK_SHIFT == 8
	subw	#1,d0		| prepare the dbra word counter
#else
	lsll	#CSHIFT-8,d0
	bra	cc_lend		| jump to dbra counter tests
cc_oloop:		| each outer loop copies 16 megabytes
	swap	d0
#endif
cc_iloop:		| each inner loop copies 256 bytes
	moveml	a0@+,d1-d7/a2-a5
	moveml	d1-d7/a2-a5,a1@		|  44 Bytes copied
	moveml	a0@+,d1-d7/a2-a5
	moveml	d1-d7/a2-a5,a1@(44)		|  88 Bytes copied
	moveml	a0@+,d1-d7/a2-a5
	moveml	d1-d7/a2-a5,a1@(88)		| 132 Bytes copied
	moveml	a0@+,d1-d7/a2-a5
	moveml	d1-d7/a2-a5,a1@(132)		| 176 Bytes copied
	moveml	a0@+,d1-d7/a2-a5
	moveml	d1-d7/a2-a5,a1@(176)		| 220 Bytes copied
	moveml	a0@+,d1-d7/a2-a3
	moveml	d1-d7/a2-a3,a1@(220)		| 256 Bytes copied

	addw	#256,a1
cc_lend:
	dbra	d0,cc_iloop		| do inner loop count adjustment
#if CLICK_SHIFT != 8
	swap	d0		| exchange high and low part of count
	dbra	d0,cc_oloop		| do outer loop count adjustment
#endif

	moveml	sp@+,d2-d7/a2-a5		| restore registers
cc_ret:
	rts	


|*===========================================================================*
|*				flipclicks				     *
|*===========================================================================*
|PUBLIC void flipclicks(phys_clicks c1, phys_clicks c2, phys_clicks n)

#define	fc_c1	4(sp)
#define	fc_c2	6(sp)
#define	fc_ncl	8(sp)

_flipclicks:
	movel	#CSHIFT,d1

	movel	#0,d0
	movew	fc_c1,d0		| source1 address in clicks
	lsll	d1,d0
	movel	d0,a0		| source1 address

	movel	#0,d0
	movew	fc_c2,d0		| source2 address in clicks
	lsll	d1,d0
	movel	d0,a1		| source2 address

	movel	#0,d0
	movew	fc_ncl,d0		| number of clicks
	beq	fc_ret		| nothing to flip
	lsll	#CSHIFT-6,d0		| we are flipping in 64 byte blocks

	moveml	d2-d7/a2-a6,sp@-		| save registers
	bra	fc_lend		| jump to end of loop for dbra

fc_oloop:		| each outer loop flips 4 megabytes
	swap	d0		| get high and low part of count right
fc_iloop:		| each loop flips 64 bytes
	moveml	a0@+,d1-d6
	moveml	a1@+,d7/a2-a6
	moveml	d1-d6,a1@(-24)
	moveml	d7/a2-a6,a0@(-24)		|  24 Bytes copied
	moveml	a0@+,d1-d6
	moveml	a1@+,d7/a2-a6
	moveml	d1-d6,a1@(-24)
	moveml	d7/a2-a6,a0@(-24)		|  48 Bytes copied
	moveml	a0@+,d1-d4
	moveml	a1@+,a2-a5
	moveml	d1-d4,a1@(-16)
	moveml	a2-a5,a0@(-16)		|  64 Bytes copied
fc_lend:
	dbra	d0,fc_iloop		| do inner loop count adjustment
	swap	d0		| exchange high and low part of count
	dbra	d0,fc_oloop		| do outer loop count adjustment

	moveml	sp@+,d2-d7/a2-a6		| restore registers

fc_ret:
	rts	


|*===========================================================================*
|*				zeroclicks				     *
|*===========================================================================*
|PUBLIC void zeroclicks(phys_clicks dest, phys_clicks nclicks)

#define	zc_dest	4(sp)
#define	zc_ncl	6(sp)

_zeroclicks:
	movel	#CSHIFT,d1


	movel	#0,d0
	movew	zc_dest,d0		| address in clicks
	addw	zc_ncl,d0		| set address to top of area to clear
	lsll	d1,d0
	movel	d0,a0

	movel	#0,d0
	movew	zc_ncl,d0		| number of clicks
	beq	zc_ret		| nothing to clear

	moveml	d2-d7/a2-a6,sp@-		| save registers

	movel	#0,d1		| clear registers
	movel	d1,d2
	movel	d1,d3
	movel	d1,d4
	movel	d1,d5
	movel	d1,d6
	movel	d1,d7
	movel	d1,a1
	movel	d1,a2
	movel	d1,a3
	movel	d1,a4
	movel	d1,a5
	movel	d1,a6

#if CLICK_SHIFT == 8
	subw	#1,d0		| prepare the dbra word counter
#else
	lsll	#CSHIFT-8,d0
	bra	zc_lend		| jump to dbra counter tests
zc_oloop:		| each outer loop clears 16 megabytes
	swap	d0
#endif
zc_iloop:		| each inner loop clears 256 bytes
	moveml	d1-d7/a1-a6,a0@-		|  52 Bytes zeroed
	moveml	d1-d7/a1-a6,a0@-		| 104 Bytes zeroed
	moveml	d1-d7/a1-a6,a0@-		| 156 Bytes zeroed
	moveml	d1-d7/a1-a6,a0@-		| 208 Bytes zeroed
	moveml	d1-d7/a1-a5,a0@-		| 256 Bytes zeroed
zc_lend:
	dbra	d0,zc_iloop		| do inner loop count adjustment
#if CLICK_SHIFT != 8
	swap	d0		| exchange high and low part of count
	dbra	d0,zc_oloop		| do outer loop count adjustment
#endif

	moveml	sp@+,d2-d7/a2-a6		| restore registers
zc_ret:
	rts	


|*===========================================================================*
|*				phys_copy				     *
|*===========================================================================*
|PUBLIC void phys_copy(phys_bytes src, phys_bytes dest, phys_bytes n)

#define	pc_src	4(sp)
#define	pc_dest	8(sp)
#define	pc_cnt	12(sp)

_phys_copy:
	movel	pc_src,a0		| load source pointer
	movel	pc_dest,a1		| load destination pointer
	movel	a0,d0
	movel	a1,d1
	eorb	d1,d0
	btst	#0,d0		| pointers mutually aligned?
	bne	ucopy
	movel	pc_cnt,d0
	beq	end		| if cnt == 0 we are finished yet
	btst	#0,d1		| pointers aligned, but odd?
	beq	check64		| no
	moveb	a0@+,a1@+		| copy odd byte
	subl	#1,d0		| decrement count
check64:
	movel	#63,d1		| +
	cmpl	d1,d0		| +
	bls	copy4		| + count < 64
	moveml	d2-d7/a2-a6,sp@-		| save regs for movem use

|==== copying 256 bytes/loop ====
	clrw	d1		| count mod 256
	moveb	d0,d1
	lsrl	#8,d0		| count div 256
	bra	end256
loop256:
	moveml	a0@+,d2-d7/a2-a6		| copy 11x4 bytes
	moveml	d2-d7/a2-a6,a1@
	moveml	a0@+,d2-d7/a2-a6		| copy 11x4 bytes
	moveml	d2-d7/a2-a6,a1@(44)
	moveml	a0@+,d2-d7/a2-a6		| copy 11x4 bytes
	moveml	d2-d7/a2-a6,a1@(88)
	moveml	a0@+,d2-d7/a2-a6		| copy 11x4 bytes
	moveml	d2-d7/a2-a6,a1@(132)
	moveml	a0@+,d2-d7/a2-a6		| copy 11x4 bytes
	moveml	d2-d7/a2-a6,a1@(176)
	moveml	a0@+,d2-d7/a2-a4		| copy  9x4 bytes
	moveml	d2-d7/a2-a4,a1@(220)
	lea	a1@(256),a1
end256:
	dbra	d0,loop256		| decrement count, test and loop
	movew	d1,d0		| remainder becomes new count
	beq	done		| more to copy? no	!

|==== copying 64 bytes/loop ====
	andw	#0x3F,d1		| count mod 64
	lsrw	#6,d0		| count div 64
	bra	end64
loop64:
	moveml	a0@+,d2-d7/a4-a5		| copy 8x4 bytes
	moveml	d2-d7/a4-a5,a1@
	moveml	a0@+,d2-d7/a4-a5		| copy 8x4 bytes
	moveml	d2-d7/a4-a5,a1@(32)
	lea	a1@(64),a1
end64:
	dbra	d0,loop64		| decrement count, test and loop
done:	moveml	sp@+,d2-d7/a2-a6		| restore regs for movem use
	movew	d1,d0		| remainder becomes new count
	beq	end

|==== copying 4 bytes/loop ====
copy4:		| NOTE: count must be max. 63
	movew	d0,d1
	andw	#3,d1
	lsrw	#2,d0		| avoid expensive dbra instruction
	addw	d0,d0		| by jumping to the appropriate
	negw	d0		| move.l instructions offset
	jmp	pc@(end4,d0:w)

	movel	a0@+,a1@+
	movel	a0@+,a1@+
	movel	a0@+,a1@+
	movel	a0@+,a1@+
	movel	a0@+,a1@+
	movel	a0@+,a1@+
	movel	a0@+,a1@+
	movel	a0@+,a1@+
	movel	a0@+,a1@+
	movel	a0@+,a1@+
	movel	a0@+,a1@+
	movel	a0@+,a1@+
	movel	a0@+,a1@+
	movel	a0@+,a1@+
	movel	a0@+,a1@+
end4:
	movew	d1,d0		| remainder becomes new count

|==== copying 1 byte/loop ====
copy1:		| NOTE: count must be max. 15!
	addw	d0,d0		| avoid expensive dbra instruction
	negw	d0		| by jumping to the appropriate
	jmp	pc@(end1,d0:w)		| move.b instructions offset

	moveb	a0@+,a1@+
	moveb	a0@+,a1@+
	moveb	a0@+,a1@+
	moveb	a0@+,a1@+
	moveb	a0@+,a1@+
	moveb	a0@+,a1@+
	moveb	a0@+,a1@+
	moveb	a0@+,a1@+
	moveb	a0@+,a1@+
	moveb	a0@+,a1@+
	moveb	a0@+,a1@+
	moveb	a0@+,a1@+
	moveb	a0@+,a1@+
	moveb	a0@+,a1@+
	moveb	a0@+,a1@+
end1:
|==== exit phys_copy ====
end:
	rts	

|==== 16 bytes/loop copy for unaligned pointers ====
ucopy:
	movel	pc_cnt,d0
| count can be big; test on it	!
	movel	#16,d1		| == moveq; 4
	cmpl	d1,d0		| 6
	blt	copy1
copy16:
	movew	d0,d1
	andw	#0x0F,d1
	lsrl	#4,d0
	bra	end16
loop16:
	moveb	a0@+,a1@+
	moveb	a0@+,a1@+
	moveb	a0@+,a1@+
	moveb	a0@+,a1@+
	moveb	a0@+,a1@+
	moveb	a0@+,a1@+
	moveb	a0@+,a1@+
	moveb	a0@+,a1@+
	moveb	a0@+,a1@+
	moveb	a0@+,a1@+
	moveb	a0@+,a1@+
	moveb	a0@+,a1@+
	moveb	a0@+,a1@+
	moveb	a0@+,a1@+
	moveb	a0@+,a1@+
	moveb	a0@+,a1@+
end16:
	dbra	d0,loop16
	subl	#0x10000,d0		| count can even be bigger (>1MB)
	bhi	loop16		|  (dbra handles only word counters)
	movew	d1,d0
	bra	copy1


.globl	loop256
.globl	loop64
.globl	copy4
.globl	copy1
.globl	ucopy
.globl	done
#endif
