/*
 * Hard disk definitions
 */

#define SCSI_SCAT_SECTORS 0
#define ACSI_SCAT_SECTORS 12

/*
 * SCSI Commands; for historical reasons the common commands start with HD_
 *	impl describes the implementation
 *		M: mandatory
 *		E: required for SCSI devices that support device-independen
 *		   self-configuring software
 *		O: optional
 *		V: vendor unique; the comment field explains this
 */

/* Group 0 commands for all device types */

/*      name 	 cmd	  impl	   comment   */
#define	HD_TUR	0x00	/* O */	/* Test Unit Ready */
#define	HD_RQS	0x03	/* M */	/* ReQuest Sense */
#define	HD_INQ	0x12	/* E */	/* INQuire */
#define	HD_CPY	0x18	/* O */	/* CoPY */
#define	HD_RDN	0x1c	/* O */	/* Receive DiagNostic */
#define	HD_SDN	0x1d	/* O */	/* Send DiagNostic */

/* Group 1 commands for all device types */

/*      name 	 cmd	  impl	   comment   */
#define	HD_CMP	0x39	/* O */	/* CoMPare */
#define	HD_CAV	0x3a	/* O */	/* Copy And Verify */

/* Group 0 commands for direct-access devices */
/* most of these apply for write-once read-multiple devices and for 
   read-only direct-access devices as well */

/*      name 	 cmd	  impl	   comment   */
#define	HD_RZU	0x01	/* O */	/* ReZero Unit */
#define	HD_FMTU	0x04	/* M */	/* ForMaT Unit */
#define	HD_REAS 0x07	/* O */	/* REASsign blocks */
#define	HD_RD	0x08	/* M */	/* ReaD */
#define	HD_WR	0x0a	/* M */	/* WRite */
#define	HD_SK	0x0b	/* O */	/* SeeK */
#define	HD_TRAN	0x0f	/* V */	/* TRANslate (Adaptec 4000a and 4520) */
#define	HD_SET	0x10	/* V */	/* Set Error Threshold (Adaptec 4520) */
#define	HD_RUC	0x11	/* V */	/* Read/reset Usage Counter (Adaptec 4520) */
#define	HD_WDB	0x13	/* V */	/* Write Data Buffer (Adaptec 4520) */
#define	HD_RDB	0x14	/* V */	/* Read Data Buffer (Adaptec 4520) */
#define	HD_MSL	0x15	/* O */	/* Mode SeLect */
#define	HD_RES	0x16	/* O */	/* REServe */
#define	HD_REL	0x17	/* O */	/* RELease */
#define	HD_MSN	0x1a	/* O */	/* Mode SeNse */
#define	HD_SSU	0x1b	/* O */	/* Start/Stop Unit (on some sytems:
				   seek to shipping position */
#define	HD_PAMR	0x1e	/* O */	/* Prevent/Allow Media Removal */

/* Group 1 commands for direct-access devices */
/* most of these apply for write-once read-multiple devices and for 
   read-only direct-access devices as well */

/*      name 	 cmd	  impl	   comment   */
#define	HD_RCAP	0x25	/* E */	/* Read CAPacity */
#define	HD_XRD	0x28	/* E */	/* eXtended ReaD */
#define	HD_XWR	0x2a	/* E */	/* eXtended WRite */
#define	HD_WAV	0x2e	/* O */	/* Write And Verify */
#define	HD_VFY	0x2f	/* O */	/* VeriFY */
#define	HD_SDH	0x30	/* O */	/* Search Data High */
#define	HD_SDE	0x31	/* O */	/* Search Data Equal */
#define	HD_SDL	0x32	/* O */	/* Search Data Low */
#define	HD_SLIM	0x33	/* O */	/* Set LIMits */

/* Group 0 commands for sequential-access devices */

/*      name 	 cmd	  impl	   comment   */
#define	SQ_REW	0x01	/* M */	/* REWind */
#define	SQ_RBL  0x05	/* E */	/* Read Block Limits */
#define	SQ_RD	0x08	/* M */	/* ReaD */
#define	SQ_WR	0x0a	/* M */	/* WRite */
#define	SQ_TSEL	0x0b	/* O */	/* Track SELect */
#define	SQ_RREV	0x0f	/* O */	/* Read Reverse */
#define	SQ_WFM	0x10	/* M */	/* Write File Marks */
#define	SQ_SPC	0x11	/* O */	/* SPaCe */
#define	SQ_VFY	0x13	/* O */	/* VeriFY */
#define	SQ_RBD	0x14	/* O */	/* Recover Buffered Data */
#define	SQ_MSL	0x15	/* O */	/* Mode SeLect */
#define	SQ_RES	0x16	/* O */	/* REServe */
#define	SQ_REL	0x17	/* O */ /* RELease */
#define	SQ_ERS	0x19	/* O */ /* ERaSe */
#define	SQ_MSN	0x1a	/* O */	/* Mode SeNse */
#define	SQ_LUNL	0x1b	/* O */	/* Load/UNLoad */
#define	SQ_PAMR	0x1e	/* O */	/* Prevent/Allow Media Removal */

/* Group 0 commands for printer devices */

/*      name 	 cmd	  impl	   comment   */
#define	PR_FMT	0x04	/* O */	/* ForMaT */
#define	PR_PRNT	0x0a	/* M */	/* PRiNT */
#define	PR_SLAP	0x0b	/* O */	/* SLew And Print */
#define	PR_FLUS	0x10	/* O */	/* FLUSh buffer */
#define	PR_RBD	0x14	/* O */	/* Recover Buffered Data */
#define	PR_MSL	0x15	/* O */	/* Mode SeLect */
#define	PR_RES	0x16	/* O */	/* REServe */
#define	PR_REL	0x17	/* O */	/* RELease */
#define	PR_MSN	0x1a	/* O */	/* Mode SeNse */
#define	PR_STP	0x1b	/* O */	/* STop Print */

/* Group 0 commands for processor devices */

/*      name 	 cmd	  impl	   comment   */
#define	UP_RCV	0x08	/* O */	/* ReCeiVe */
#define	UP_SND	0x0a	/* M */	/* SeND */


/*
 * bits in completion status byte:
 */
#define	HDC_CC		0x02	/* Check Condition */
#define	HDC_BUSY	0x08	/* Check Condition */

/*
 * codes for byte 0 of status block returned by HD_RQS:
 */
#define	NOERROR		0x00
#define	NOINDEX		0x01
#define	NOSEEKCOMPLETE	0x02
#define	WRITEFAULT	0x03
#define	DRIVENOTREADY	0x04
#define	NOTRACKZERO	0x06
#define	IDECC		0x10
#define	BADDATA		0x11
#define	NOIDADDRMARK	0x12
#define	NODATAADDRMARK	0x13
#define	NORECORD	0x14
#define	BADSEEK		0x15
#define	DATANOECC	0x18
#define	DATAECC		0x19
#define	BADBLOCK	0x1a
#define	BADFORMAT	0x1c
#define	BADSELFTEST	0x1d
#define	BADOP		0x20
#define	BADBLOCKADDR	0x21
#define	BADFINALADDR	0x23
#define	BADARG		0x24
#define	BADDEVICE	0x25
#define	BADPARITY	0x26
#define	CARTCHANGED	0x28
#define	ERRORCNTOVFLW	0x2c
#define	SELFTESTFAILED	0x30

/*
 * Partition information (part of sector 0)
 */
struct pi {
  char	pi_flag;		/* active byte */
  char	pi_id[3];		/* partition id (GEM or MIX(?)) */
  long	pi_start;		/* first sector */
  long	pi_size;		/* total number of sectors */
};

/* values of pi_flag: */
#define	PI_INVALID	0x00
#define	PI_VALID	0x01
#define	PI_BOOT		0x81

#define	NPARTS		4		/* normal # partitions */

#ifndef PARTITIONING
#define PARTITIONING ATARI
#endif

/*
 * Hard disk information (sector 0) of any hard disk drive
 */
struct hi {
#if (PARTITIONING == ATARI)
# define	ADDPARTS	0 	/* additional # extended partitions */
  char		hi_gap1[0x1B6];		/* reserved */
#endif /* ATARI */

#if (PARTITIONING == SUPRA)
# define	ADDPARTS	8	/* additional # extended partitions */
  char		hi_gap1[0x156];		/* reserved */
  struct pi	hd_addpi[ADDPARTS];		/* 8 supra partition descriptors */
#endif /* SUPRA */

#if (PARTITIONING == EICKMANN )
# define	ADDPARTS	4	/* additional # extended partitions */
  char		hi_gap1[0x180];		/* reserved			*/
  int		hi_defpart;		/* def. partition (auto-folder)	*/
  struct pi	hd_addpi[ADDPARTS];	/* 4 partition descriptors	*/
  int		hi_reserved;
  int		hi_maxsubdirs;		/* not used by MINIX		*/
#endif /* EICKMANN */

#if (PARTITIONING == CBHD)
# define	ADDPARTS	8 	/* additional # extended partitions */
  char		hi_gap1[0x151];		/* reserved			*/
  char		hi_version;		/* version of rootsector format */
  char		hi_lable[4];		/* == "CBHD"			*/
  struct pi	hd_addpi[ADDPARTS];	/* 8 partition descriptors	*/
#endif /* CBHD */

  short		hi_cc;		/* number of cylinders */
  char		hi_dhc;		/* number of heads */
  char		hi_gap2[1];	/* reserved */
  short		hi_rwcc;	/* reduced write current cyl */
  short		hi_wpc;		/* write-precompensation cyl */
  char		hi_lz;		/* landing zone */
  char		hi_rt;		/* seek rate code */
  char		hi_in;		/* interleave factor */
  char		hi_spt;		/* sectors per track */
  long		hd_size;	/* size of disk in sectors */
  struct pi	hd_pi[NPARTS];	/* 4 partition descriptors */
  long		bsl_start;	/* start of bad sector table */
  long		bsl_count;	/* number of bad sectors */
  short		hi_magic;	/* reserved for checksum */
};
