#include <errno.h>
#include <lib.h>
#include <limits.h>
#include <unistd.h>

#define CHUNKSIZE (32768-1024)

ssize_t write(int fd, const void *buffer, size_t nbytes)
{
  message m;
  ssize_t count = 0;
  int chunk, value;

  if (nbytes > SSIZE_MAX)
  {
	errno = EINVAL;
	return -1L;
  }

  while (nbytes > 0)
  {
    chunk = (nbytes > CHUNKSIZE) ? CHUNKSIZE : nbytes;
    nbytes -= chunk;
    m.m1_i1 = fd;
    m.m1_i2 = chunk;
    m.m1_p1 = (char *) buffer;
    count += value = _syscall(FS, WRITE, &m);
    if (value != chunk)
    {
      if (value == -1)
        return -1L;
      else
        break;
    }
    buffer += value;
  }

  return count;
}
